@@ -36,7 +36,7 @@
   #include <mach/mach.h>
   #include <time.h>
   #include <sys/time.h>
-#elif defined(__unix__)
+#elif defined(__unix__) || defined(__HAIKU__)
   #include <unistd.h>
   #if defined(__GLIBC__)
     #include <time.h>
@@ -171,7 +171,7 @@ void init_buffer(void *src, int size, int rshift) {
 
 
 void do_bench(char *compressor, char *shuffle, int nthreads, int size, int elsize,
-              int rshift, FILE * ofile) {
+              int rshift, int unsafe, FILE * ofile) {
   void *src, *srccpy;
   void *dest[NCHUNKS], *dest2;
   int nbytes = 0, cbytes = 0;
@@ -190,6 +190,7 @@ void do_bench(char *compressor, char *shuffle, int nthreads, int size, int elsiz
   else if (strcmp(shuffle, "noshuffle") == 0) {
       doshuffle = BLOSC_NOSHUFFLE;
     }
+  else abort();
 
   blosc_set_nthreads(nthreads);
   if(blosc_set_compressor(compressor) < 0){
@@ -201,7 +202,9 @@ void do_bench(char *compressor, char *shuffle, int nthreads, int size, int elsiz
   /* Initialize buffers */
   srccpy = malloc(size);
   retcode = posix_memalign( (void **)(&src), 32, size);
+  if (retcode) abort();
   retcode = posix_memalign( (void **)(&dest2), 32, size);
+  if (retcode) abort();
 
   /* zero src to initialize byte on it, and not only multiples of 4 */
   memset(src, 0, size);
@@ -209,9 +212,11 @@ void do_bench(char *compressor, char *shuffle, int nthreads, int size, int elsiz
   memcpy(srccpy, src, size);
   for (j = 0; j < nchunks; j++) {
      retcode = posix_memalign( (void **)(&dest[j]), 32, size+BLOSC_MAX_OVERHEAD);
+     if (retcode) abort();
   }
 
-  fprintf(ofile, "--> %d, %d, %d, %d, %s, %s\n", nthreads, size, elsize, rshift, compressor, shuffle);
+  fprintf(ofile, "--> %d, %d, %d, %d, %s, %s, %s\n", nthreads, size, elsize,
+          rshift, compressor, shuffle, unsafe ? "unsafe" : "safe");
   fprintf(ofile, "********************** Run info ******************************\n");
   fprintf(ofile, "Blosc version: %s (%s)\n", BLOSC_VERSION_STRING, BLOSC_VERSION_DATE);
   fprintf(ofile, "Using synthetic data with %d significant bits (out of 32)\n", rshift);
@@ -278,7 +283,8 @@ void do_bench(char *compressor, char *shuffle, int nthreads, int size, int elsiz
           nbytes = size;
         }
         else {
-          nbytes = blosc_decompress(dest[j], dest2, size);
+          nbytes = unsafe ? blosc_decompress_unsafe(dest[j], dest2, size)
+                          : blosc_decompress(dest[j], dest2, size);
         }
       }
     }
@@ -377,6 +383,7 @@ int main(int argc, char *argv[]) {
   int rshift = 19;                      /* Significant bits */
   int workingset = 256*MB;              /* The maximum allocated memory */
   int nthreads_, size_, elsize_, rshift_, i;
+  int unsafe = 1;
   FILE * output_file = stdout;
   blosc_timestamp_t last, current;
   float totaltime;
@@ -387,7 +394,7 @@ int main(int argc, char *argv[]) {
   strncpy(usage, "Usage: bench [blosclz | lz4 | lz4hc | snappy | zlib | zstd] "
           "[noshuffle | shuffle | bitshuffle] "
           "[single | suite | hardsuite | extremesuite | debugsuite] "
-          "[nthreads] [bufsize(bytes)] [typesize] [sbits]", 255);
+          "[nthreads] [bufsize(bytes)] [typesize] [sbits] [unsafe | safe]", 255);
 
   if (argc < 2) {
     printf("%s\n", usage);
@@ -485,7 +492,18 @@ int main(int argc, char *argv[]) {
     rshift = atoi(argv[7]);
   }
 
-  if ((argc >= 9) || !(single || suite || hard_suite || extreme_suite)) {
+  if (argc >= 9) {
+    if (strcmp(argv[8], "unsafe") == 0) {
+      unsafe = 1;
+    } else if (strcmp(argv[8], "safe") == 0) {
+      unsafe = 0;
+    } else {
+      printf("%s\n", usage);
+      exit(1);
+    }
+  }
+
+  if ((argc >= 10) || !(single || suite || hard_suite || extreme_suite)) {
     printf("%s\n", usage);
     exit(1);
   }
@@ -497,7 +515,7 @@ int main(int argc, char *argv[]) {
 
   if (suite) {
     for (nthreads_=1; nthreads_ <= nthreads; nthreads_++) {
-      do_bench(compressor, shuffle, nthreads_, size, elsize, rshift, output_file);
+      do_bench(compressor, shuffle, nthreads_, size, elsize, rshift, unsafe, output_file);
     }
   }
   else if (hard_suite) {
@@ -512,7 +530,7 @@ int main(int argc, char *argv[]) {
             nchunks = get_nchunks(size_+i, workingset);
     	    niter = 1;
             for (nthreads_ = 1; nthreads_ <= nthreads; nthreads_++) {
-              do_bench(compressor, shuffle, nthreads_, size_+i, elsize_, rshift_, output_file);
+              do_bench(compressor, shuffle, nthreads_, size_+i, elsize_, rshift_, unsafe, output_file);
               blosc_set_timestamp(&current);
               totaltime = (float)getseconds(last, current);
               printf("Elapsed time:\t %6.1f s.  Processed data: %.1f GB\n",
@@ -531,7 +549,7 @@ int main(int argc, char *argv[]) {
           for (size_ = 32*KB; size_ <= size; size_ *= 2) {
             nchunks = get_nchunks(size_+i, workingset);
             for (nthreads_ = 1; nthreads_ <= nthreads; nthreads_++) {
-              do_bench(compressor, shuffle, nthreads_, size_+i, elsize_, rshift_, output_file);
+              do_bench(compressor, shuffle, nthreads_, size_+i, elsize_, rshift_, unsafe, output_file);
               blosc_set_timestamp(&current);
               totaltime = (float)getseconds(last, current);
               printf("Elapsed time:\t %6.1f s.  Processed data: %.1f GB\n",
@@ -550,7 +568,7 @@ int main(int argc, char *argv[]) {
           for (size_ = size; size_ <= 16*MB; size_ *= 2) {
             nchunks = get_nchunks(size_+i, workingset);
             for (nthreads_ = nthreads; nthreads_ <= 6; nthreads_++) {
-              do_bench(compressor, shuffle, nthreads_, size_+i, elsize_, rshift_, output_file);
+              do_bench(compressor, shuffle, nthreads_, size_+i, elsize_, rshift_, unsafe, output_file);
               blosc_set_timestamp(&current);
               totaltime = (float)getseconds(last, current);
               printf("Elapsed time:\t %6.1f s.  Processed data: %.1f GB\n",
@@ -563,7 +581,7 @@ int main(int argc, char *argv[]) {
   }
   /* Single mode */
   else {
-    do_bench(compressor, shuffle, nthreads, size, elsize, rshift, output_file);
+    do_bench(compressor, shuffle, nthreads, size, elsize, rshift, unsafe, output_file);
   }
 
   /* Print out some statistics */
