@@ -11,12 +11,11 @@
 ########################################################################
 
 """Here is defined the AttributeSet class."""
-from __future__ import absolute_import
 
 import re
 import sys
 import warnings
-import six.moves.cPickle
+import pickle
 import numpy
 
 from . import hdf5extension
@@ -27,8 +26,6 @@ from .path import check_attribute_name
 from .undoredo import attr_to_shadow
 from .filters import Filters
 
-from six.moves import map
-import six
 
 
 # System attributes
@@ -150,9 +147,9 @@ class AttributeSet(hdf5extension.AttributeSet, object):
 
     .. rubric:: Notes on AttributeSet methods
 
-    Note that this class overrides the __getattr__(), __setattr__() and
-    __delattr__() special methods.  This allows you to read, assign or
-    delete attributes on disk by just using the next constructs::
+    Note that this class overrides the __getattr__(), __setattr__(),
+    __delattr__() and __dir__() special methods.  This allows you to
+    read, assign or delete attributes on disk by just using the next constructs::
 
         leaf.attrs.myattr = 'str attr'    # set a string (native support)
         leaf.attrs.myattr2 = 3            # set an integer (native support)
@@ -169,6 +166,10 @@ class AttributeSet(hdf5extension.AttributeSet, object):
 
     Use whatever idiom you prefer to access the attributes.
 
+    Finally, on interactive python sessions you may get autocompletions of
+    attributes named as *valid python identifiers* by pressing the  `[Tab]`
+    key, or to use the dir() global function.
+
     If an attribute is set on a target node that already has a large
     number of attributes, a PerformanceWarning will be issued.
 
@@ -282,6 +283,15 @@ class AttributeSet(hdf5extension.AttributeSet, object):
         elif attrset == "all":
             return self._v_attrnames[:]
 
+    def __dir__(self):
+        """Autocomplete only children named as valid python identifiers.
+
+        Only PY3 supports this special method.
+        """
+        return list(set(c for c in
+                    super(AttributeSet, self).__dir__() + self._v_attrnames
+                    if c.isidentifier()))
+
     def __getattr__(self, name):
         """Get the attribute named "name"."""
 
@@ -313,18 +323,18 @@ class AttributeSet(hdf5extension.AttributeSet, object):
             # This format was used during the first 1.2 releases, just
             # for string defaults.
             try:
-                retval = six.moves.cPickle.loads(value)
+                retval = pickle.loads(value)
                 retval = numpy.array(retval)
             except ImportError:
                 retval = None  # signal error avoiding exception
-        elif maybe_pickled and name == 'FILTERS' and format_version < (2, 0):
+        elif maybe_pickled and name == 'FILTERS' and format_version is not None and format_version < (2, 0):
             # This is a big hack, but we don't have other way to recognize
             # pickled filters of PyTables 1.x files.
             value = _old_filters_re.sub(_new_filters_sub, value, 1)
-            retval = six.moves.cPickle.loads(value)  # pass unpickling errors through
+            retval = pickle.loads(value)  # pass unpickling errors through
         elif maybe_pickled:
             try:
-                retval = six.moves.cPickle.loads(value)
+                retval = pickle.loads(value)
             # except cPickle.UnpicklingError:
             # It seems that pickle may raise other errors than UnpicklingError
             # Perhaps it would be better just an "except:" clause?
@@ -337,10 +347,10 @@ class AttributeSet(hdf5extension.AttributeSet, object):
                 # unplicked as bytestrings. Hence try 'latin1' first.
                 # Ref: http://bugs.python.org/issue6784
                 try:
-                    retval = six.moves.cPickle.loads(value, encoding='latin1')
+                    retval = pickle.loads(value, encoding='latin1')
                 except TypeError:
                     try:
-                        retval = six.moves.cPickle.loads(value, encoding='bytes')
+                        retval = pickle.loads(value, encoding='bytes')
                     except:
                         retval = value
                 except:
@@ -357,24 +367,17 @@ class AttributeSet(hdf5extension.AttributeSet, object):
                 # explaining how the user can tell where the problem was.
                 retval = value
             # Additional check for allowing a workaround for #307
-            if isinstance(retval, six.text_type) and retval == u'':
+            if isinstance(retval, str) and retval == u'':
                 retval = numpy.array(retval)[()]
-        elif name == 'FILTERS' and format_version >= (2, 0):
+        elif name == 'FILTERS' and format_version is not None and format_version >= (2, 0):
             retval = Filters._unpack(value)
         elif name == 'TITLE' and not isinstance(value, str):
-            if sys.version_info[0] < 3:
-                # unicode is OK for TITLE
-                retval = value
-            else:
-                retval = value.decode('utf-8')
-        elif (issysattrname(name) and isinstance(value, (bytes, six.text_type)) and
+            retval = value.decode('utf-8')
+        elif (issysattrname(name) and isinstance(value, (bytes, str)) and
               not isinstance(value, str) and not _field_fill_re.match(name)):
             # system attributes should always be str
-            if sys.version_info[0] < 3:
-                retval = value.encode()
-            else:
-                # python 3, bytes and not "FIELD_[0-9]+_FILL"
-                retval = value.decode('utf-8')
+            # python 3, bytes and not "FIELD_[0-9]+_FILL"
+            retval = value.decode('utf-8')
         else:
             retval = value
 
@@ -403,17 +406,17 @@ class AttributeSet(hdf5extension.AttributeSet, object):
             elif name == "NROWS":
                 stvalue = numpy.array(value, dtype=SizeType)
                 value = stvalue[()]
-            elif name == "FILTERS" and self._v__format_version >= (2, 0):
+            elif name == "FILTERS" and self._v__format_version is not None and self._v__format_version >= (2, 0):
                 stvalue = value._pack()
                 # value will remain as a Filters instance here
         # Convert value from a Python scalar into a NumPy scalar
         # (only in case it has not been converted yet)
         # Fixes ticket #59
         if (stvalue is value and
-                type(value) in (bool, bytes, int, float, complex, six.text_type,
+                type(value) in (bool, bytes, int, float, complex, str,
                                 numpy.unicode_)):
             # Additional check for allowing a workaround for #307
-            if isinstance(value, six.text_type) and len(value) == 0:
+            if isinstance(value, str) and len(value) == 0:
                 stvalue = numpy.array(u'')
             else:
                 stvalue = numpy.array(value)
