@@ -11,7 +11,6 @@
 ########################################################################
 
 """Here is defined the Group class."""
-from __future__ import absolute_import
 
 import os
 import re
@@ -31,7 +30,6 @@ from .leaf import Leaf
 from .unimplemented import UnImplemented, Unknown
 
 from .link import Link, SoftLink, ExternalLink
-import six
 
 
 obversion = "1.0"
@@ -42,7 +40,6 @@ class _ChildrenDict(ProxyDict):
         return container._f_get_child(key)
 
 
-@six.python_2_unicode_compatible
 class Group(hdf5extension.Group, Node):
     """Basic PyTables grouping structure.
 
@@ -83,10 +80,6 @@ class Group(hdf5extension.Group, Node):
     ----------
     parentnode
         The parent :class:`Group` object.
-
-        .. versionchanged:: 3.0
-           Renamed from *parentNode* to *parentnode*
-
     name : str
         The name of this node in its parent group.
     title
@@ -97,6 +90,9 @@ class Group(hdf5extension.Group, Node):
         A Filters instance
 
 
+    .. versionchanged:: 3.0
+       *parentNode* renamed into *parentnode*
+
     Notes
     -----
     The following documentation includes methods that are automatically
@@ -105,8 +101,8 @@ class Group(hdf5extension.Group, Node):
     For instance, this class defines the __setattr__, __getattr__,
     __delattr__ and __dir__ methods, and they set, get and delete
     *ordinary Python attributes* as normally intended. In addition to that,
-     __getattr__ allows getting *child nodes* by their name for the sake of
-     easy interaction on the command line, as long as there is no Python
+    __getattr__ allows getting *child nodes* by their name for the sake of
+    easy interaction on the command line, as long as there is no Python
     attribute with the same name. Groups also allow the interactive
     completion (when using readline) of the names of child nodes.
     For instance::
@@ -662,7 +658,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
             (srcparent, dstparent) = parentstack.pop()
 
             if use_hardlinks:
-                for srcchild in six.itervalues(srcparent._v_children):
+                for srcchild in srcparent._v_children.values():
                     addr, rc = srcchild._get_obj_info()
                     if rc > 1 and addr in address_map:
                         where, name = address_map[addr][0]
@@ -689,7 +685,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
                                 (dstparent._v_pathname, srcchild.name)
                             ]
             else:
-                for srcchild in six.itervalues(srcparent._v_children):
+                for srcchild in srcparent._v_children.values():
                     dstchild = srcchild._g_copy_as_child(dstparent, **kwargs)
                     if isinstance(srcchild, Group):
                         parentstack.append((srcchild, dstchild))
@@ -741,22 +737,22 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
 
         if not classname:
             # Returns all the children alphanumerically sorted
-            names = sorted(six.iterkeys(self._v_children))
+            names = sorted(self._v_children.keys())
             for name in names:
                 yield self._v_children[name]
         elif classname == 'Group':
             # Returns all the groups alphanumerically sorted
-            names = sorted(six.iterkeys(self._v_groups))
+            names = sorted(self._v_groups.keys())
             for name in names:
                 yield self._v_groups[name]
         elif classname == 'Leaf':
             # Returns all the leaves alphanumerically sorted
-            names = sorted(six.iterkeys(self._v_leaves))
+            names = sorted(self._v_leaves.keys())
             for name in names:
                 yield self._v_leaves[name]
         elif classname == 'Link':
             # Returns all the links alphanumerically sorted
-            names = sorted(six.iterkeys(self._v_links))
+            names = sorted(self._v_links.keys())
             for name in names:
                 yield self._v_links[name]
         elif classname == 'IndexArray':
@@ -766,7 +762,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
             class_ = get_class_by_name(classname)
 
             children = self._v_children
-            childnames = sorted(six.iterkeys(children))
+            childnames = sorted(children.keys())
 
             for childname in childnames:
                 childnode = children[childname]
@@ -790,7 +786,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
         # Iterate over the descendants
         while stack:
             objgroup = stack.pop()
-            groupnames = sorted(six.iterkeys(objgroup._v_groups))
+            groupnames = sorted(objgroup._v_groups.keys())
             # Sort the groups before delivering. This uses the groups names
             # for groups in tree (in order to sort() can classify them).
             for groupname in groupnames:
@@ -1051,7 +1047,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
         if use_hardlinks:
             address_map = kwargs.setdefault('address_map', {})
 
-            for child in six.itervalues(self._v_children):
+            for child in self._v_children.values():
                 addr, rc = child._get_obj_info()
                 if rc > 1 and addr in address_map:
                     where, name = address_map[addr][0]
@@ -1074,7 +1070,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
                             (dstparent._v_pathname, child.name)
                         ]
         else:
-            for child in six.itervalues(self._v_children):
+            for child in self._v_children.values():
                 child._f_copy(dstparent, None, overwrite, recursive, **kwargs)
 
 
@@ -1114,7 +1110,7 @@ be ready to see PyTables asking for *lots* of memory and possibly slow I/O."""
 
         rep = [
             '%r (%s)' % (childname, child.__class__.__name__)
-            for (childname, child) in six.iteritems(self._v_children)
+            for (childname, child) in self._v_children.items()
         ]
         childlist = '[%s]' % (', '.join(rep))
 
